;-----------------------------------------------------------------------------
; Definition of PDL2000-Functions for usage in AutoIt3
;
; Author: Dr. Oliver Merk - PeCon GmbH - Germany
; Version 01.000 / 24.01.2012
;
; PDL2000.dll and SiUSBXp.dll must be in the same directory like this script
;
;-----------------------------------------------------------------------------

; ################################################
; #### Definition of device specific commands ####
;#################################################


Func GetActualValue ($device, $channel)
; Usage:
; $return = GetActualValue ($device, $channel)
;	$device: device handle
;	$channel: channel number
;
;	returns temperature in C [e.g. 36.8]
;	return value 409.6 = nothing connected
;
	local $strucvalue = DllStructCreate("SHORT");
	local $return = DllCall($PDL2000_DLL, "ULONG", "_PdlGetActualValue@12", "ULONG", $device, "byte", $channel, "ptr", DllStructGetPtr($strucvalue))
	
	if $DEBUG = True Then
		ConsoleWrite("DllCall(PdlGetActualValue): err =" & @error & @LF) 
		ConsoleWrite("DllCall(PdlGetActualValue): ret =" & $return[0] & @LF)
	EndIf
	
	Return DllStructGetData($strucvalue, 1)/10
	
EndFunc

;-----------------------------------------------------------------------------

Func GetSetpointValue ($device, $channel)
; Usage:
; $return = GetSetpointValue ($device, $channel)
;	$device: device handle
;	$channel: channel number
;
;	returns temperature in C [e.g. 36.8]
;
	local $Strucvalue = DllStructCreate("SHORT")
	local $return = DllCall($PDL2000_DLL, "ULONG", "_PdlGetSetpointValue@12", "ULONG", $device, "byte", $channel, "ptr", DllStructGetPtr($strucvalue))
	
	if $DEBUG = True Then
		ConsoleWrite("DllCall(PdlSetSetpointValue): err =" & @error & @LF) 
		ConsoleWrite("DllCall(PdlSetSetpointValue): ret =" & $return[0] & @LF)
	EndIf
	
	Return DllStructGetData($strucvalue, 1)/10
	
EndFunc

;-----------------------------------------------------------------------------

Func SetSetpointValue ($device, $channel, $value)
; Usage:
; SetSetpointValue ($device, $channel, $value)
;	$device: device handle
;	$channel: channel number
;	$value: temperature in C [e.g. 36.8]
;
	local $Strucvalue = DllStructCreate("SHORT")
	DllStructSetData($strucvalue,1,$value*10)
	local $return = DllCall($PDL2000_DLL, "ULONG", "_PdlSetSetpointValue@12", "ULONG", $device, "byte", $channel, "ptr", DllStructGetPtr($strucvalue))

	if $DEBUG = True Then
		ConsoleWrite("DllCall(PdlSetSetpointValue): err =" & @error & @LF) 
		ConsoleWrite("DllCall(PdlSetSetpointValue): ret =" & $return[0] & @LF)
	EndIf
	
EndFunc

;-----------------------------------------------------------------------------

Func GetLoopControl ($device, $channel)
; Usage:
; GetLoopControl ($device, $channel)
;	$device: device handle
;	$channel: channel number
;
;	returns 0 = Off / 1 = On
;
	local $Strucvalue = DllStructCreate("BYTE")
	local $return = DllCall($PDL2000_DLL, "ULONG", "_PdlGetLoopControl@12", "ULONG", $device, "byte", $channel, "ptr", DllStructGetPtr($strucvalue))
	
	if $DEBUG = True Then
		ConsoleWrite("DllCall(PdlSetLoopControl): err =" & @error & @LF) 
		ConsoleWrite("DllCall(PdlSetLoopControl): ret =" & $return[0] & @LF)
	EndIf

	Return DllStructGetData($strucvalue, 1)
	
EndFunc

;-----------------------------------------------------------------------------

Func SetLoopControl ($device, $channel, $value)
; Usage:
; SetLoopControl ($device, $channel, $value)
;	$device: device handle
;	$channel: channel number
;	$value: 0 = Off / 1 = On
;
	local $Strucvalue = DllStructCreate("BYTE")
	DllStructSetData($strucvalue,1,$value)
	local $return = DllCall($PDL2000_DLL, "ULONG", "_PdlSetLoopControl@12", "ULONG", $device, "byte", $channel, "ptr", DllStructGetPtr($strucvalue))
	
	if $DEBUG = True Then
		ConsoleWrite("DllCall(PdlSetLoopControl): err =" & @error & @LF) 
		ConsoleWrite("DllCall(PdlSetLoopControl): ret =" & $return[0] & @LF)
	EndIf
	
EndFunc

;-----------------------------------------------------------------------------

Func GetComponentName ($device, $channel)
; Usage:
; $return = GetComponentName ($device, $channel)
;	$device: device handle
;	$channel: channel number
;
;	returns name of configured component
;
	local $Strucvalue = DllStructCreate("char[12]")
	local $return = DllCall($PDL2000_DLL, "ULONG", "_PdlGetComponentName@16", "ULONG", $device, "byte", $channel, "ptr", DllStructGetPtr($strucvalue), "byte", 12)
	
	if $DEBUG = True Then
		ConsoleWrite("DllCall(PdlSetLoopControl): err =" & @error & @LF) 
		ConsoleWrite("DllCall(PdlSetLoopControl): ret =" & $return[0] & @LF)
	EndIf

	Return DllStructGetData($strucvalue, 1)

EndFunc

;-----------------------------------------------------------------------------








;-----------------------------------------------------------------------------
; ###############################################
; #### Definition of higher level commands ####
;################################################



;-----------------------------------------------------------------------------

Func RampSetpointValue ($device, $channel, $startvalue, $endvalue, $time, $ticks)
; Usage:
;  RampSetpointValue ($device, $channel, $startvalue, $endvalue, $time, $interval)
;	$device: device handle
;	$channel: channel number
;	$startvalue: first setpoint value
;	$endvalue: last setpoint value
;	$time: time from first to last setpoint value in seconds
;	$ticks: number of interpolated setpoints send to device during ramp
;

	For $counter = 0 To $ticks
	
		SetSetpointValue ($device, $channel, $startvalue + ($endvalue - $startvalue)*$counter/$ticks )
		Sleep($time/$ticks*1000)
	
	Next


	
	
EndFunc

;-----------------------------------------------------------------------------










; ###############################################
; #### Definition of administrative commands ####
;################################################

; Get decimal delimiter (region specific)
; Can be used for text-output of data for easier excel-import
global $decimalchar = RegRead("HKCU\Control Panel\International", "sDecimal")

;-----------------------------------------------------------------------------

; Format comma separated value with one decimal place according to your region settings
Func FormatDecimalValue ($value)
	Local $value1 = StringReplace(StringFormat("%02.1f", $value), ".", $decimalchar)
	
	Return $value1
EndFunc

;-----------------------------------------------------------------------------

; Open DLL and get handle
Func OpenPeConDll ()
	Local $PDL2000_DLL = DllOpen("PDL2000.dll")
	If $PDL2000_DLL = -1 And $DEBUG = True Then ConsoleWrite("DLL could not be opened")
	
	Return $PDL2000_DLL

EndFunc

;-----------------------------------------------------------------------------

; Close Dll
; Handle to Dll is needed as reference
Func ClosePeConDll ($PDL2000_DLL)
	DllClose($PDL2000_DLL)
EndFunc

;-----------------------------------------------------------------------------

; Get device handle for first PeCon device found on USB
;
; This function checks all USB devices for a match with $serialnumber
; Therefore "PdlEnumNext" is called until an error is returned
; The correct handle is determined by using the serial number in the structure $DevInfo
;
Func OpenPeConDevice ($serialnumber)
	; Set up structures
	$DevInfo = DllStructCreate("ptr name;ptr serial;ptr internal");
	$Handle = DllStructCreate("ULONG");

	; Reset PDL2000.dll internal enumeration 
	$return = DllCall($PDL2000_DLL, "ULONG", "_PdlEnumReset@0")
	if $DEBUG = True Then
		ConsoleWrite("DllCall(PdlEnumReset): err=" & @error & @LF)
		ConsoleWrite("DllCall(PdlEnumReset): ret=" & $return[0] & @LF)
	EndIf

	; Get info for each PeCon USB device and check if $serialnumber matches
	; A maximum of 100 devices is scanned
	local $count = 0
	Do
	
		$count = $count + 1
		
		; Get device infos
		$return = DllCall($PDL2000_DLL, "ULONG", "_PdlEnumNext@4", "ptr", DllStructGetPtr($DevInfo))
		
		; Set up structures
		$namestruc = DllStructCreate("char name[64]", 		DllStructGetData($DevInfo, "name"));
		$serialstruc = DllStructCreate("char serial[11]",	DllStructGetData($DevInfo, "serial"));
		
		if $DEBUG = True Then
			ConsoleWrite("DllCall(PdlEnumNext): err =" & @error & @LF) 
			ConsoleWrite("DllCall(PdlEnumNext): ret =" & $return[0] & @LF)
			ConsoleWrite("DllCall(PdlEnumNext): nam =" & DllStructGetData($namestruc, "name") & @LF)
			ConsoleWrite("DllCall(PdlEnumNext): ser =" & DllStructGetData($serialstruc, "serial") & @LF)
		EndIf

		; Check, if $serialnumber matches
		If $serialnumber == DllStructGetData($serialstruc, "serial")  Then 
			$count = 200 ; 200 means "device found"
		EndIf

		; Clear device infos for next loop run
		DllStructSetData($namestruc, 1, "")
		DllStructSetData($serialstruc, 1, "")
		
	Until $return[0] <> 0 Or $count > 100
	
	If $count = 200 Then
		; Open Device and get handle
		$return = DllCall($PDL2000_DLL, "ULONG", "_PdlOpen@8", "ptr", DllStructGetPtr($DevInfo), "ptr", DllStructGetPtr($Handle))
		if $DEBUG = True Then
			ConsoleWrite("DllCall(PdlOpen): err =" & @error & @LF) 
			ConsoleWrite("DllCall(PdlOpen): ret =" & $return[0] & @LF)
		EndIf
		$Hnd = DllStructGetData($Handle, 1)
		If $DEBUG = True Then
			ConsoleWrite ("Found Device: " & $serialnumber & @LF)
		EndIf

	Else
		; Notify user that device is not found and invalidate handle
		msgbox(48, "OpenPeconDevice Error", "Device " & $serialnumber & " not found!" & @LF & "Check serial number.")
		$Hnd = -1
		If $DEBUG = True Then
			ConsoleWrite ("Device not found: " & $serialnumber & @LF)
		EndIf
	EndIf
	

	; Free PDL2000.dll internal data from enumeration
	$return = DllCall($PDL2000_DLL, "ULONG", "_PdlEnumFree@0")
	if $DEBUG = True Then
		ConsoleWrite("DllCall(PdlEnumFree): err =" & @error & @LF) 
		ConsoleWrite("DllCall(PdlEnumFree): ret =" & $return[0] & @LF)
	EndIf
	
	Return $Hnd

EndFunc

;-----------------------------------------------------------------------------

; Get Dll Version
; Gives back PDL2000.dll version string
Func GetDllVersion ()
	$dllversion =  DllStructCreate("ULONG")
	$return = DllCall($PDL2000_DLL, "ULONG", "_PdlGetVersion@4", "ULONG_PTR", DllStructGetPtr($dllversion))
	if $DEBUG = True Then
		ConsoleWrite("DllCall(PdlGetVersion): err=" & @error & @LF)
		ConsoleWrite("DllCall(PdlGetVersion): ret=" & $return[0] & @LF)
	EndIf
	
	$dllversion = Hex(DllStructGetData($dllversion, 1))
	
	Return StringLeft($dllversion,2) & "." & StringMid($dllversion,3,2) & "." & StringRight($dllversion,4)

EndFunc
